package org.apithefire.test;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.easymock.IExpectationSetters;
import org.easymock.classextension.EasyMock;

public class Mock<T> {
	
	private Class<T> clazz;
	private List<Method> methods;

	private Mock(Class<T> clazz) {
		if (clazz == null) {
			throw new NullPointerException();
		}
		this.clazz = clazz;
	}
	
	public static <T> T create(Class<T> clazz) {
		return EasyMock.createMock(clazz);
	}
	
	public static <T> T createNice(Class<T> clazz) {
		return EasyMock.createNiceMock(clazz);
	}
	
	public static <T> T createStrict(Class<T> clazz) {
		return EasyMock.createStrictMock(clazz);
	}
	
	public static <T> Mock<T> forClass(Class<T> clazz) {
		return new Mock<T>(clazz);
	}
	
	public Mock<T> method(String name, Class<?>... parameterTypes) {
		if (clazz.isInterface()) {
			throw new TestException(
					"Can not selectively mock methods of an interface.");
		}
		try {
			Method method = clazz.getDeclaredMethod(name, parameterTypes);
			
			method.setAccessible(true);
			
			if (methods == null) {
				methods = new ArrayList<Method>();
			}
			methods.add(method);
			return this;
		} catch (NoSuchMethodException e) {
			throw new TestException(e);
		}
	}
	
	public T create() {
		if (methods == null) {
			return EasyMock.createMock(clazz);
		}
		return EasyMock.createMock(clazz, methods.toArray(new Method[0]));
	}
	
	public T createNice() {
		if (methods == null) {
			return EasyMock.createNiceMock(clazz);
		}
		return EasyMock.createNiceMock(clazz, methods.toArray(new Method[0]));
	}
	
	public T createStrict() {
		if (methods == null) {
			return EasyMock.createStrictMock(clazz);
		}
		return EasyMock.createStrictMock(clazz, methods.toArray(new Method[0]));
	}
	
    public static <T> IExpectationSetters<T> expect(T value) {
        return EasyMock.expect(value);
    }

    public static IExpectationSetters<Object> expect() {
        return EasyMock.expectLastCall();
    }
    
    public static void replay(Object mock) {
    	EasyMock.replay(mock);
    }
    
    public static void replay(Object... mocks) {
    	for (Object mock : mocks) {
    		EasyMock.replay(mock);
    	}
    }
    
    public static void verify(Object mock) {
    	EasyMock.verify(mock);
    }
    
    public static void verify(Object... mocks) {
    	for (Object mock : mocks) {
    		EasyMock.verify(mock);
    	}
    }
    
    public static Object anyObject() {
    	return EasyMock.anyObject();
    }
	
}
